﻿using Microsoft.Practices.ServiceLocation;
using Syncfusion.Windows.Tools.Controls;
using SyncfusionDockingDispose.ViewModels;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Automation.Peers;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace SyncfusionDockingDispose
{
    /// <summary>
    /// Interaction logic for DockingAdapter.xaml
    /// </summary>
    public partial class DockingAdapter : UserControl
    {
        public DockingAdapter()
        {
            InitializeComponent();
            dockingManager.Loaded += dockingManager_Loaded;
            dockingManager.DockStateChanged += dockingManager_DockStateChanged;

            dockingManager.TabClosed += DockingManager_TabClosed;
        }

        /// <summary>
        /// Removing this callback did not force the FakeUserControlPeer to be accessed.  It does 
        /// seem to clear several anchors to the view model, but the drag drop handler is still anchored.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DockingManager_TabClosed(object sender, CloseTabEventArgs e)
        {
            e.TargetTabItem.DataContext = null; 
        }

        private void dockingManager_Loaded(object sender, RoutedEventArgs e)
        {
            ((DocumentContainer)dockingManager.DocContainer).AddTabDocumentAtLast = true;
        }

        private void dockingManager_DockStateChanged(FrameworkElement sender, DockStateEventArgs e)
        {
            if (e.NewState == DockState.Hidden)
            {
                var view = sender as FrameworkElement;
                var viewModel = view.DataContext as ViewModelBase;

                viewModel.OnRequestClose(); // request removal from the collection at the VM level.  Collection changed handles the removal from the dockingManager.Children.

                if (ActiveDocument == viewModel)
                    ActiveDocument = null;
                view.DataContext = null; // should force the removal of any anchors to command predicates
                viewModel.Dispose();
            }
        }

        #region Active Document Dependency Property

        public ViewModelBase ActiveDocument
        {
            get { return (ViewModelBase)GetValue(ActiveDocumentProperty); }
            set { SetValue(ActiveDocumentProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ActiveDocument.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ActiveDocumentProperty =
            DependencyProperty.Register("ActiveDocument", typeof(ViewModelBase), typeof(DockingAdapter), new PropertyMetadata(null, new PropertyChangedCallback(onActiveDocumentChanged)));

        private static void onActiveDocumentChanged(DependencyObject sender, DependencyPropertyChangedEventArgs args)
        {
            DockingAdapter adapter = sender as DockingAdapter;
            if (adapter != null)
            {
                foreach (FrameworkElement element in adapter.dockingManager.Children)
                {
                    if (element.DataContext == args.NewValue)
                    {
                        adapter.dockingManager.ActiveWindow = element;
                        break;
                    }
                }
            }
        }

        #endregion

        #region ItemsSource Dependency Property

        public IList ItemsSource
        {
            get { return (IList)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ItemsSource.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ItemsSourceProperty =
            DependencyProperty.Register("ItemsSource", typeof(IList), typeof(DockingAdapter), new PropertyMetadata(null));

        #endregion

        protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
        {
            if (e.Property.Name == "ItemsSource")
            {
                if (e.OldValue != null)
                {
                    var oldcollection = e.OldValue as INotifyCollectionChanged;
                    oldcollection.CollectionChanged -= CollectionChanged;
                }

                if (e.NewValue != null)
                {
                    ((DocumentContainer)dockingManager.DocContainer).AddTabDocumentAtLast = true;
                    var newcollection = e.NewValue as INotifyCollectionChanged;
                    foreach (ViewModelBase viewModel in ((IList)e.NewValue))
                    {
                        FrameworkElement view = viewLocator(viewModel);
                        DockingManager.SetHeader(view, viewModel.Header);
                        DockingManager.SetState(view, DockState.Document);
                        dockingManager.Children.Add(view);
                    }
                    newcollection.CollectionChanged += CollectionChanged;
                }
            }
            base.OnPropertyChanged(e);
        }

        private void CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.OldItems != null)
            {
                foreach (var item in e.OldItems)
                {
                    FrameworkElement control = (from FrameworkElement element in dockingManager.Children
                                                where element.DataContext == item
                                                select element).FirstOrDefault();
                    dockingManager.Children.Remove(control);
                }
            }

            if (e.NewItems != null)
            {
                foreach (ViewModelBase viewModel in e.NewItems)
                {
                    FrameworkElement view = viewLocator(viewModel);
                    DockingManager.SetHeader(view, viewModel.Header);
                    DockingManager.SetState(view, DockState.Document);
                    dockingManager.Children.Add(view);
                }
            }
        }

        private void dockingManager_ActiveWindowChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var content = e.NewValue as FrameworkElement;
            if (content == null)
                return;

            ViewModelBase dataContext = content.DataContext as ViewModelBase;
            if (dataContext != null)
                ActiveDocument = (ViewModelBase)content.DataContext;
        }

        private FrameworkElement viewLocator(object viewModel)
        {
            try
            {
                var viewTypeName = viewModel.GetType().FullName.Replace("ViewModels", "Views").Replace("ViewModel", "");
                var viewType = viewModel.GetType().Assembly.GetType(viewTypeName);

                var view = (FrameworkElement)ServiceLocator.Current.TryResolve(viewType);
                if (view == null)
                    throw new ActivationException(string.Format("Unable to resolve view for {0}.", viewTypeName));

                view.DataContext = viewModel;
                return view;
            }
            catch (Exception ex)
            {
                throw new ActivationException("Unable to resolve view.", ex);
            }
        }

        /// <summary>
        /// this does not seem to be called.
        /// </summary>
        /// <returns></returns>
        protected override AutomationPeer OnCreateAutomationPeer()
        {
            return new FakeUserControlPeer(this);
        }
    }
}
